@@ -62,13 +62,29 @@ namespace Opal {
       PCLASSINFO(subscriber, PThread);
 
     public:
-      subscriber (const Opal::Account & _account,
+      subscriber (std::string _username,
+		  std::string _host,
+		  std::string _authentication_username,
+		  std::string _password,
+		  bool _is_enabled,
+		  SIPRegister::CompatibilityModes _compat_mode,
+		  unsigned _timeout,
+		  std::string _aor,
                   Opal::Sip::EndPoint & _manager,
-		  bool _registering)
+		  bool _registering,
+                  const PSafePtr<OpalPresentity> & _presentity)
         : PThread (1000, AutoDeleteThread),
-	  account (_account),
+	  username(_username),
+	  host(_host),
+	  authentication_username(_authentication_username),
+	  password(_password),
+	  is_enabled(_is_enabled),
+	  compat_mode(_compat_mode),
+	  timeout(_timeout),
+	  aor(_aor),
 	  manager (_manager),
-	  registering (_registering)
+	  registering (_registering),
+	  presentity (_presentity)
       {
         this->Resume ();
       };
@@ -77,23 +93,30 @@ namespace Opal {
       {
 	if (registering) {
 
-	  manager.Register (account.get_username (),
-			    account.get_host (),
-			    account.get_authentication_username (),
-			    account.get_password (),
-			    account.is_enabled (),
-			    account.is_limited (),
-			    account.get_timeout ());
-	} else {
+          if (presentity && !presentity->IsOpen ())
+            presentity->Open ();
+	  manager.Register (username, host, authentication_username, password, is_enabled, compat_mode, timeout);
+	}
+        else {
+	  manager.Unregister (aor);
 
-	  manager.Unregister (account.get_aor ());
+          if (presentity && presentity->IsOpen ())
+            presentity->Close ();
 	}
       };
 
     private:
-      const Opal::Account & account;
+      std::string username;
+      std::string host;
+      std::string authentication_username;
+      std::string password;
+      bool is_enabled;
+      SIPRegister::CompatibilityModes compat_mode;
+      unsigned timeout;
+      std::string aor;
       Opal::Sip::EndPoint & manager;
       bool registering;
+      const PSafePtr<OpalPresentity> & presentity;
     };
   };
 };
@@ -109,7 +132,6 @@ Opal::Sip::EndPoint::EndPoint (Opal::CallManager & _manager,
 	core (_core)
 {
   boost::shared_ptr<Ekiga::ChatCore> chat_core = core.get<Ekiga::ChatCore> ("chat-core");
-  boost::shared_ptr<Opal::Bank> bank = core.get<Opal::Bank> ("opal-account-store");
 
   auto_answer_call = false;
   protocol_name = "sip";
@@ -119,10 +141,6 @@ Opal::Sip::EndPoint::EndPoint (Opal::CallManager & _manager,
   dialect = boost::shared_ptr<SIP::Dialect>(new SIP::Dialect (core, boost::bind (&Opal::Sip::EndPoint::send_message, this, _1, _2)));
   chat_core->add_dialect (dialect);
 
-  bank->account_added.connect (boost::bind (&Opal::Sip::EndPoint::on_bank_updated, this, _1));
-  bank->account_removed.connect (boost::bind (&Opal::Sip::EndPoint::on_bank_updated, this, _1));
-  bank->account_updated.connect (boost::bind (&Opal::Sip::EndPoint::on_bank_updated, this, _1));
-
   /* Timeouts */
   SetAckTimeout (PTimeInterval (0, 32));
   SetPduCleanUpTimeout (PTimeInterval (0, 1));
@@ -142,13 +160,12 @@ Opal::Sip::EndPoint::EndPoint (Opal::CallManager & _manager,
   manager.AddRouteEntry("pc:.* = sip:<da>");
 
   /* NAT Binding */
-  SetNATBindingRefreshMethod (SIPEndPoint::EmptyRequest);
+  SetNATBindingRefreshMethod (SIPEndPoint::Options);
 }
 
 
 Opal::Sip::EndPoint::~EndPoint ()
 {
-  manager.ShutDownEndpoints ();
 }
 
 
@@ -176,8 +193,10 @@ Opal::Sip::EndPoint::menu_builder_add_actions (const std::string& fullname,
 					       Ekiga::MenuBuilder & builder)
 {
   bool populated = false;
+  boost::shared_ptr<Opal::Bank> bk = bank.lock ();
 
-  boost::shared_ptr<Opal::Bank> bank = core.get<Opal::Bank> ("opal-account-store");
+  if (!bk)
+    return false;
 
   std::list<std::string> uris;
   std::list<std::string> accounts_list;
@@ -187,8 +206,8 @@ Opal::Sip::EndPoint::menu_builder_add_actions (const std::string& fullname,
 
   if (uri.find ("@") == string::npos) {
 
-    for (Opal::Bank::iterator it = bank->begin ();
-	 it != bank->end ();
+    for (Opal::Bank::iterator it = bk->begin ();
+	 it != bk->end ();
 	 it++) {
 
       if ((*it)->get_protocol_name () == "SIP" && (*it)->is_enabled ()) {
@@ -237,10 +256,10 @@ Opal::Sip::EndPoint::menu_builder_add_actions (const std::string& fullname,
     }
 
     if (0 == GetConnectionCount ())
-      builder.add_action ("call", call_action.str (),
+      builder.add_action ("phone-pick-up", call_action.str (),
                           boost::bind (&Opal::Sip::EndPoint::on_dial, this, (*it)));
     else
-      builder.add_action ("transfer", transfer_action.str (),
+      builder.add_action ("call-transfer", transfer_action.str (),
                           boost::bind (&Opal::Sip::EndPoint::on_transfer, this, (*it)));
 
     ita++;
@@ -257,7 +276,7 @@ Opal::Sip::EndPoint::menu_builder_add_actions (const std::string& fullname,
     else
       msg_action << _("Message");
 
-    builder.add_action ("message", msg_action.str (),
+    builder.add_action ("im-message-new", msg_action.str (),
                         boost::bind (&Opal::Sip::EndPoint::on_message, this, (*it), fullname));
 
     ita++;
@@ -274,10 +293,11 @@ Opal::Sip::EndPoint::send_message (const std::string & _uri,
 				   const std::string & _message)
 {
   if (!_uri.empty () && (_uri.find ("sip:") == 0 || _uri.find (':') == string::npos) && !_message.empty ()) {
-
-    PURL fromAddress;
-    PString conversationId;
-    Message (PURL(_uri), "text/plain;charset=UTF-8", _message, fromAddress, conversationId);
+    OpalIM im;
+    im.m_to = PURL (_uri);
+    im.m_mimeType = "text/plain;charset=UTF-8";
+    im.m_body = _message;
+    Message (im);
     return true;
   }
 
@@ -430,6 +450,7 @@ Opal::Sip::EndPoint::get_outbound_proxy () const
 void
 Opal::Sip::EndPoint::set_nat_binding_delay (unsigned delay)
 {
+  PTRACE (3, "Ekiga\tNat binding delay set to " << delay);
   if (delay > 0)
     SetNATBindingTimeout (PTimeInterval (0, delay));
 }
@@ -456,23 +477,45 @@ Opal::Sip::EndPoint::get_aor_domain (const std::string & aor)
 
 
 bool
-Opal::Sip::EndPoint::subscribe (const Opal::Account & account)
+Opal::Sip::EndPoint::subscribe (const Opal::Account & account,
+                                const PSafePtr<OpalPresentity> & presentity)
 {
   if (account.get_protocol_name () != "SIP")
     return false;
 
-  new subscriber (account, *this, true);
+  new subscriber (account.get_username (),
+		  account.get_host (),
+		  account.get_authentication_username (),
+		  account.get_password (),
+		  account.is_enabled (),
+		  account.get_compat_mode (),
+		  account.get_timeout (),
+		  account.get_aor (),
+		  *this,
+                  true,
+                  presentity);
   return true;
 }
 
 
 bool
-Opal::Sip::EndPoint::unsubscribe (const Opal::Account & account)
+Opal::Sip::EndPoint::unsubscribe (const Opal::Account & account,
+                                  const PSafePtr<OpalPresentity> & presentity)
 {
   if (account.get_protocol_name () != "SIP")
     return false;
 
-  new subscriber (account, *this, false);
+  new subscriber (account.get_username (),
+		  account.get_host (),
+		  account.get_authentication_username (),
+		  account.get_password (),
+		  account.is_enabled (),
+		  account.get_compat_mode (),
+		  account.get_timeout (),
+		  account.get_aor (),
+		  *this,
+                  false,
+                  presentity);
   return true;
 }
 
@@ -483,7 +526,7 @@ Opal::Sip::EndPoint::Register (const std::string username,
 			       const std::string auth_username,
 			       const std::string password,
 			       bool is_enabled,
-			       bool is_limited,
+			       SIPRegister::CompatibilityModes compat_mode,
 			       unsigned timeout)
 {
   PString _aor;
@@ -501,13 +544,12 @@ Opal::Sip::EndPoint::Register (const std::string username,
   SIPRegister::Params params;
   params.m_addressOfRecord = PString (aor.str ());
   params.m_registrarAddress = PString (host_);
-  if (is_limited)
-    params.m_contactAddress = "%LIMITED";
+  params.m_compatibility = compat_mode;
   params.m_authID = auth_username;
   params.m_password = password;
   params.m_expire = is_enabled ? timeout : 0;
-  params.m_minRetryTime = 0;
-  params.m_maxRetryTime = 0;
+  params.m_minRetryTime = PMaxTimeInterval;  // use default value
+  params.m_maxRetryTime = PMaxTimeInterval;  // use default value
 
   // Register the given aor to the give registrar
   if (!SIPEndPoint::Register (params, _aor))
@@ -902,9 +944,9 @@ Opal::Sip::EndPoint::OnMESSAGECompleted (const SIPMessage::Params & params,
   std::string uri = (const char*) to.AsString ();
   std::string display_name = (const char*) to.GetDisplayName ();
 
-  std::string reason_shown = _("Could not send message");
+  std::string reason_shown = _("Could not send message: ");
   if (reason == SIP_PDU::Failure_TemporarilyUnavailable)
-    reason_shown += _(": user offline");
+    reason_shown += _("user offline");
   else
     reason_shown += SIP_PDU::GetStatusCodeDescription (reason);  // too many to translate them with _()...
 
@@ -987,36 +1029,18 @@ void Opal::Sip::EndPoint::on_transfer (std::string uri)
 }
 
 
-void
-Opal::Sip::EndPoint::on_bank_updated (Ekiga::AccountPtr /*account*/)
-{
-  boost::shared_ptr<Opal::Bank> bank = core.get<Opal::Bank> ("opal-account-store");
-  bank->visit_accounts (boost::bind (&Opal::Sip::EndPoint::visit_accounts, this, _1));
-}
-
-
-bool
-Opal::Sip::EndPoint::visit_accounts (Ekiga::AccountPtr account_)
-{
-  Opal::AccountPtr account = boost::dynamic_pointer_cast<Opal::Account> (account_);
-
-  PWaitAndSignal m(aorMutex);
-  accounts[account->get_host ()] = account->get_aor ();
-
-  return true;
-}
-
-
 void
 Opal::Sip::EndPoint::registration_event_in_main (const std::string aor,
 						 Opal::Account::RegistrationState state,
 						 const std::string msg)
 {
-  boost::shared_ptr<Opal::Bank> bank = core.get<Opal::Bank> ("opal-account-store");
-  AccountPtr account = bank->find_account (aor);
+  if (boost::shared_ptr<Opal::Bank> bk = bank.lock ()) {
 
-  if (account)
-    account->handle_registration_event (state, msg);
+    AccountPtr account = bk->find_account (aor);
+
+    if (account)
+      account->handle_registration_event (state, msg);
+  }
 }
 
 void
@@ -1039,11 +1063,63 @@ void
 Opal::Sip::EndPoint::mwi_received_in_main (const std::string aor,
 					   const std::string info)
 {
-  boost::shared_ptr<Opal::Bank> bank = core.get<Opal::Bank> ("opal-account-store");
-  AccountPtr account = bank->find_account (aor);
+  if (boost::shared_ptr<Opal::Bank> bk = bank.lock ()) {
 
-  if (account) {
+    AccountPtr account = bk->find_account (aor);
 
-    account->handle_message_waiting_information (info);
+    if (account) {
+
+      account->handle_message_waiting_information (info);
+    }
   }
 }
+
+void
+Opal::Sip::EndPoint::update_bank ()
+{
+  bank = core.get<Opal::Bank> ("opal-account-store");
+  if (boost::shared_ptr<Opal::Bank> bk = bank.lock ()) { // should always happen, but still
+
+    bk->account_added.connect (boost::bind (&Opal::Sip::EndPoint::account_added, this, _1));
+    bk->account_updated.connect (boost::bind (&Opal::Sip::EndPoint::account_updated_or_removed, this, _1));
+    bk->account_removed.connect (boost::bind (&Opal::Sip::EndPoint::account_updated_or_removed, this, _1));
+    account_updated_or_removed (Ekiga::AccountPtr ()/* unused*/);
+  }
+}
+
+void
+Opal::Sip::EndPoint::account_updated_or_removed (Ekiga::AccountPtr /*account*/)
+{
+  /* we don't remember what the account information was, so we need
+   * to clear our current information everytime something changed
+   * (hopefully nobody has hundreds of opal accounts that get updated
+   * often, so performance shouldn't be an issue!
+   */
+
+  { // keep the mutex only to clear the accounts variable...
+    PWaitAndSignal m(aorMutex);
+    accounts.clear ();
+  }
+  { // ... because here we call something which will want that very same mutex!
+    bank = core.get<Opal::Bank> ("opal-account-store");
+    if (boost::shared_ptr<Opal::Bank> bk = bank.lock ()) { // should always happen, but still
+
+      bk->visit_accounts (boost::bind (&Opal::Sip::EndPoint::visit_account, this, _1));
+    }
+  }
+}
+
+bool
+Opal::Sip::EndPoint::visit_account (Ekiga::AccountPtr _account)
+{
+  account_added (_account);
+  return true;
+}
+
+void
+Opal::Sip::EndPoint::account_added (Ekiga::AccountPtr _account)
+{
+  Opal::AccountPtr account = boost::dynamic_pointer_cast<Opal::Account> (_account);
+  PWaitAndSignal m(aorMutex);
+  accounts[account->get_host ()] = account->get_aor ();
+}
